@@ -43,6 +43,10 @@
 #include <poll.h>
 #endif
 
+#if defined(HAVE_SYS_UIO_H) || (defined(__APPLE__) && defined(__MACH__))
+#include <sys/uio.h>
+#endif
+
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -233,7 +237,6 @@ rpc_which_events(struct rpc_context *rpc)
 int
 rpc_write_to_socket(struct rpc_context *rpc)
 {
-	int32_t count;
 	struct rpc_pdu *pdu;
         int ret = 0;
         
@@ -250,25 +253,47 @@ rpc_write_to_socket(struct rpc_context *rpc)
         }
 #endif /* HAVE_MULTITHREADING */
 	while ((pdu = rpc->outqueue.head) != NULL) {
-		int64_t total;
+                struct iovec iov[RPC_MAX_VECTORS];
+                struct iovec *tmpiov;
+                size_t num_done = pdu->out.num_done;
+                int niov = pdu->out.niov;
+                int i;
+                ssize_t count;
 
-		total = pdu->outdata.size;
+                for (i = 0; i < niov; i++) {
+                        iov[i].iov_base = pdu->out.iov[i].buf;
+                        iov[i].iov_len = pdu->out.iov[i].len;
+                }
+                tmpiov = iov;
 
-		count = send(rpc->fd, pdu->outdata.data + pdu->written,
-                             (int)(total - pdu->written), MSG_NOSIGNAL);
-		if (count == -1) {
-			if (errno == EAGAIN || errno == EWOULDBLOCK) {
-				ret = 0;
-                                goto finished;
-			}
-			rpc_set_error(rpc, "Error when writing to socket :%s"
-                                      "(%d)", strerror(errno), errno);
-			ret = -1;
+                /* Skip the vectors we have alredy written */
+                while (num_done >= tmpiov->iov_len) {
+                        num_done -= tmpiov->iov_len;
+                        tmpiov++;
+                        niov--;
+                }
+
+                /* Adjust the first vector to send */
+                tmpiov->iov_base = (char *)tmpiov->iov_base + num_done;
+                tmpiov->iov_len -= num_done;
+
+                count = writev(rpc->fd, tmpiov, niov);
+                if (count == -1) {
+                        if (errno == EAGAIN || errno == EWOULDBLOCK) {
+                                ret = 0;
+                                 goto finished;
+
+                        }
+                        rpc_set_error(rpc, "Error when writing to "
+                                      "socket :%d %s", errno,
+                                      rpc_get_error(rpc));
+                        ret = -1;
                         goto finished;
-		}
+                }
 
-		pdu->written += count;
-		if (pdu->written == total) {
+                pdu->out.num_done += count;
+
+		if (pdu->out.num_done == pdu->out.total_size) {
 			unsigned int hash;
 
 			rpc->outqueue.head = pdu->next;
@@ -281,7 +306,7 @@ rpc_write_to_socket(struct rpc_context *rpc)
                                 goto finished;
                         }
 
-			hash = rpc_hash_xid(pdu->xid);
+			hash = rpc_hash_xid(rpc, pdu->xid);
 			rpc_enqueue(&rpc->waitpdu[hash], pdu);
 			rpc->waitpdu_len++;
 		}
@@ -341,29 +366,30 @@ rpc_read_from_socket(struct rpc_context *rpc)
                  * 4 bytes at the beginning of every pdu.
                  */
 		if (rpc->inpos < 4) {
-			buf = (void *)rpc->rm_buf;
 			pdu_size = 4;
 		} else {
-			pdu_size = rpc_get_pdu_size((void *)&rpc->rm_buf);
-			if (rpc->inbuf == NULL) {
-				if (pdu_size > NFS_MAX_XFER_SIZE + 4096) {
-					rpc_set_error(rpc, "Incoming PDU "
-                                                      "exceeds limit of %d "
-                                                      "bytes.",
-                                                      NFS_MAX_XFER_SIZE + 4096);
-					return -1;
-				}
-				rpc->inbuf = malloc(pdu_size);
-				if (rpc->inbuf == NULL) {
-					rpc_set_error(rpc, "Failed to allocate "
-                                                      "buffer of %d bytes for "
-                                                      "pdu, errno:%d. Closing "
-                                                      "socket.",
-                                                      (int)pdu_size, errno);
-					return -1;
-				}
-				memcpy(rpc->inbuf, &rpc->rm_buf, 4);
+			pdu_size = rpc_get_pdu_size(rpc->inbuf);
+		}
+		if (rpc->inbuf_size < pdu_size) {
+			if (pdu_size > NFS_MAX_XFER_SIZE + 4096) {
+				rpc_set_error(rpc, "Incoming PDU "
+												  "exceeds limit of %d "
+												  "bytes.",
+												  NFS_MAX_XFER_SIZE + 4096);
+				return -1;
 			}
+			buf = realloc(rpc->inbuf, pdu_size);
+			if (buf == NULL) {
+				rpc_set_error(rpc, "Failed to allocate "
+												  "buffer of %d bytes for "
+												  "pdu, errno:%d. Closing "
+												  "socket.",
+												  (int)pdu_size, errno);
+				return -1;
+			}
+			rpc->inbuf_size = pdu_size;
+			rpc->inbuf = buf;
+		} else {
 			buf = rpc->inbuf;
 		}
 
@@ -390,17 +416,14 @@ rpc_read_from_socket(struct rpc_context *rpc)
 		}
 
 		if (rpc->inpos == pdu_size) {
-			rpc->inbuf  = NULL;
 			rpc->inpos  = 0;
 
 			if (rpc_process_pdu(rpc, buf, pdu_size) != 0) {
 				rpc_set_error(rpc, "Invalid/garbage pdu "
                                               "received from server. Closing "
                                               "socket");
-				free(buf);
 				return -1;
 			}
-			free(buf);
 		}
 	} while (rpc->is_nonblocking && rpc->waitpdu_len > 0);
 
@@ -428,6 +451,14 @@ rpc_timeout_scan(struct rpc_context *rpc)
 	uint64_t t = rpc_current_time();
 	unsigned int i;
 
+        /*
+         * Only scan once per second.
+         */
+        if (t <= rpc->last_timeout_scan + 1000) {
+                return;
+        }
+        rpc->last_timeout_scan = t;
+
 #ifdef HAVE_MULTITHREADING
         if (rpc->multithreading_enabled) {
                 nfs_mt_mutex_lock(&rpc->rpc_mutex);
@@ -453,7 +484,7 @@ rpc_timeout_scan(struct rpc_context *rpc)
 			NULL, pdu->private_data);
 		rpc_free_pdu(rpc, pdu);
 	}
-	for (i = 0; i < HASHES; i++) {
+	for (i = 0; i < rpc->num_hashes; i++) {
 		struct rpc_queue *q;
 
                 q = &rpc->waitpdu[i];
@@ -897,26 +928,25 @@ rpc_reconnect_requeue(struct rpc_context *rpc)
 	rpc->is_connected = 0;
 
 	if (rpc->outqueue.head) {
-		rpc->outqueue.head->written = 0;
+		rpc->outqueue.head->out.num_done = 0;
 	}
 
 	/* Socket is closed so we will not get any replies to any commands
 	 * in flight. Move them all over from the waitpdu queue back to the
          * out queue.
 	 */
-        printf("reconnect reset waitpdu queues\n");
 #ifdef HAVE_MULTITHREADING
         if (rpc->multithreading_enabled) {
                 nfs_mt_mutex_lock(&rpc->rpc_mutex);
         }
 #endif /* HAVE_MULTITHREADING */
-	for (i = 0; i < HASHES; i++) {
+	for (i = 0; i < rpc->num_hashes; i++) {
 		struct rpc_queue *q = &rpc->waitpdu[i];
 		for (pdu = q->head; pdu; pdu = next) {
 			next = pdu->next;
 			rpc_return_to_queue(&rpc->outqueue, pdu);
 			/* we have to re-send the whole pdu again */
-			pdu->written = 0;
+			pdu->out.num_done = 0;
 		}
 		rpc_reset_queue(q);
 	}
@@ -1017,12 +1047,18 @@ rpc_set_udp_destination(struct rpc_context *rpc, char *addr, int port,
 		return -1;
  	}
 
+	rpc->is_broadcast = is_broadcast;
+	setsockopt(rpc->fd, SOL_SOCKET, SO_BROADCAST, (char *)&is_broadcast, sizeof(is_broadcast));
+
 	memcpy(&rpc->udp_dest, ai->ai_addr, ai->ai_addrlen);
 	freeaddrinfo(ai);
 
-	rpc->is_broadcast = is_broadcast;
-	setsockopt(rpc->fd, SOL_SOCKET, SO_BROADCAST, (char *)&is_broadcast,
-                   sizeof(is_broadcast));
+        if (!is_broadcast) {
+                if (connect(rpc->fd, (struct sockaddr *)&rpc->udp_dest, sizeof(rpc->udp_dest)) != 0 && errno != EINPROGRESS) {
+                        rpc_set_error(rpc, "connect() to UDP address failed. %s(%d)", strerror(errno), errno);
+                        return -1;
+                }
+        }
 
 	return 0;
 }
